/*
 * Copyright (c) 2011 Carman Consulting, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wicketopia.viewer.component;

import org.apache.wicket.Component;
import org.apache.wicket.behavior.IBehavior;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.IModel;
import org.metastopheles.PropertyMetaData;
import org.wicketopia.builder.ViewerBuilder;
import org.wicketopia.viewer.PropertyViewerProvider;

public class LabelPropertyViewer extends Label implements ViewerBuilder
{
//----------------------------------------------------------------------------------------------------------------------
// Fields
//----------------------------------------------------------------------------------------------------------------------
    public static final String TYPE_NAME = "label";
    private static final PropertyViewerProvider provider = new Provider();

//----------------------------------------------------------------------------------------------------------------------
// Static Methods
//----------------------------------------------------------------------------------------------------------------------

    public static PropertyViewerProvider getProvider()
    {
        return provider;
    }

//----------------------------------------------------------------------------------------------------------------------
// Constructors
//----------------------------------------------------------------------------------------------------------------------

    public LabelPropertyViewer(String id, IModel<?> model)
    {
        super(id, model);
    }

//----------------------------------------------------------------------------------------------------------------------
// PropertyViewer Implementation
//----------------------------------------------------------------------------------------------------------------------

    @Override
    public void addBehavior(IBehavior behavior)
    {
        add(behavior);
    }

    @Override
    public Component build()
    {
        return this;
    }

    @Override
    public void visible(boolean viewable)
    {
        setVisible(viewable);
    }

//----------------------------------------------------------------------------------------------------------------------
// Inner Classes
//----------------------------------------------------------------------------------------------------------------------

    private static class Provider implements PropertyViewerProvider
    {
        @Override
        public ViewerBuilder createPropertyViewer(String componentId, PropertyMetaData propertyMetadata, IModel<?> propertyModel)
        {
            return new LabelPropertyViewer(componentId, propertyModel);
        }
    }
}
